// Versie 5
import junit.framework.TestCase;

public class DoolhofTest extends TestCase {
  private final int RIJEN = 6, KOLOMMEN = 9;
  private Veld[][] bord;
  private Doolhof doolhof;
  
  
  public void setUp() {
    bord = new Veld[ RIJEN ][ KOLOMMEN ];
    for( int r = 0; r < RIJEN; r++ ) {
      for( int k = 0; k < KOLOMMEN; k++ ) {
        bord[ r ][ k ] = new Veld( r, k );
        if( r == 0 || r == RIJEN - 1 || k == 0 || k == KOLOMMEN -1 )
          bord[r][k].setMuur();
      }
    }
    bord[2][2].setKist( true );
    bord[2][4].setEindveld();
    bord[2][1].setSpeler( true );

    doolhof = new Doolhof( bord );
  }
  
  public void testBord() {
    Veld[][] bord = doolhof.getBord();
    System.out.println();
    for( int r = 0; r < bord.length; r++ ) {
      for( int k = 0; k < bord[0].length; k++ ) {
        Veld veld = bord[ r][k];
        if( veld.isMuur() )
          System.out.print( 'M' );
        else
        if( veld.heeftSpeler() )
          System.out.print( 'S' );
        else
        if( veld.heeftKist() )
          System.out.print( 'K' );
        else
        if( veld.isEindveld() )
          System.out.print( 'E' );
        else
          System.out.print( ' ' );
      }
      System.out.println();
    }
  }
  
  public void testSpelerpositie() {
    Positie spelerPos = doolhof.getSpelerpositie();
    assertEquals( spelerPos.getR(), 2 );
    assertEquals( spelerPos.getK(), 1 );
  }
  
  public void testSpelerpositieTijdensZetten() {
    doolhof.doeZet( Richting.NOORD );
    assertFalse( bord[2][1].heeftSpeler() );
    assertTrue( bord[1][1].heeftSpeler() );
    
    // Ga nogmaals naar noorden; er staat een muur, 
    // dus speler kan zich niet in die richting verplaatsen als het goed is
    doolhof.doeZet( Richting.NOORD );
    assertFalse( bord[2][1].heeftSpeler() );
    assertTrue( bord[1][1].heeftSpeler() );

    doolhof.doeZet( Richting.OOST );
    assertFalse( bord[1][1].heeftSpeler() );
    assertTrue( bord[1][2].heeftSpeler() );
        
    doolhof.doeZet( Richting.ZUID );
    assertFalse( bord[1][2].heeftSpeler() );
    assertTrue( bord[2][2].heeftSpeler() );
    
    doolhof.doeZet( Richting.ZUID );
    assertFalse( bord[2][2].heeftSpeler() );
    assertTrue( bord[3][2].heeftSpeler() );
    
    // Ga nogmaals naar zuiden; er staat een kist die tegen een muur staat
    // dus speler kan zich niet verplaatsen in die richting als het goed is
    doolhof.doeZet( Richting.ZUID );
    assertFalse( bord[4][2].heeftSpeler() );
    assertTrue( bord[3][2].heeftSpeler() );

    doolhof.doeZet( Richting.WEST );
    assertFalse( bord[3][2].heeftSpeler() );
    assertTrue( bord[3][1].heeftSpeler() );
  }
  
  public void testKistpositieTijdensZetten() {
    assertTrue( bord[2][2].heeftKist() );

    doolhof.doeZet( Richting.NOORD );
    assertTrue( bord[2][2].heeftKist() );
    
    doolhof.doeZet( Richting.OOST );
    assertTrue( bord[2][2].heeftKist() );
        
    doolhof.doeZet( Richting.ZUID );
    assertFalse( bord[2][2].heeftKist() );
    assertTrue( bord[3][2].heeftKist() );
    
    doolhof.doeZet( Richting.WEST );
    assertTrue( bord[3][2].heeftKist() );
  }

  public void testIsSpelKlaar() {
    doolhof.doeZet( Richting.OOST );
    assertFalse( doolhof.isSpelKlaar() );
    
    doolhof.doeZet( Richting.OOST );
    assertTrue( doolhof.isSpelKlaar() );
  }

  public static void main( String[] args ) {
    junit.swingui.TestRunner.run( DoolhofTest.class );
  }  
}
