// MVC1

import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.util.*;

public class KubusApplicatie4 extends JFrame {
  public KubusApplicatie4( JPanel view ) {
    setContentPane( view );
    setSize( 300, 150 );
    setTitle( "Kubusapplicatie 4" );
    setDefaultCloseOperation( EXIT_ON_CLOSE );
    setVisible( true );
  }
  
  public static void main( String[] args ) {
    Kubus model = new Kubus( 0 );
    new Controller( model );
  }
}



class Controller implements ActionListener {
  private Kubus model;
  private TekstView view;
  private JFrame frame;
  
  public Controller( Kubus model ) {
    this.model = model;
    view = new TekstView();
    view.setListener( this );
    model.addObserver( view );
    frame = new KubusApplicatie4( view );
  }
  
  public void actionPerformed( ActionEvent e ) {
    JTextField zijdeVak = (JTextField) e.getSource();
    String s = zijdeVak.getText();
    try {
      double zijde = Double.parseDouble( s );
      model.setZijde( zijde );
    }   
    catch( NumberFormatException ex ) {
      JOptionPane.showMessageDialog(
        frame, "Invoer " + s + " is geen getal",
        "Fout in invoer", JOptionPane.WARNING_MESSAGE );
    }
  }
}



class TekstView extends JPanel implements Observer {
  private JTextField zijdeVak, inhoudVak, oppervlakteVak;

  public TekstView() {
    Box labelBox = Box.createVerticalBox();
    labelBox.add( new JLabel( "Zijde" ) );
    labelBox.add( new JLabel( "Inhoud" ) );
    labelBox.add( new JLabel( "Oppervlakte" ) );

    zijdeVak = new JTextField( 10 );

    inhoudVak = new JTextField( 10 );
    inhoudVak.setEditable( false );

    oppervlakteVak = new JTextField( 10 );
    oppervlakteVak.setEditable( false );
    
    Box tekstvakBox = Box.createVerticalBox();
    tekstvakBox.add( zijdeVak );
    tekstvakBox.add( inhoudVak );
    tekstvakBox.add( oppervlakteVak );
    
    Box totaalBox = Box.createHorizontalBox();
    totaalBox.add( labelBox );
    totaalBox.add( Box.createHorizontalStrut( 20 ) );
    totaalBox.add( tekstvakBox );

    setBorder( new EmptyBorder( 20, 20, 20, 20 ) );
    add( totaalBox );
  }

  public void update( Observable o, Object arg ) {
    Kubus model = (Kubus) o; 
    inhoudVak.setText( String.format( "%8.2f", model.getInhoud() ) );
    oppervlakteVak.setText( String.format( "%8.2f", model.getOppervlakte() ) );
  }
  
  public void setListener( ActionListener listener ) {
    zijdeVak.addActionListener( listener );
  }
}

class Kubus extends Observable {
  private double zijde;
  
  public Kubus( double zijde ) {
    this.zijde = zijde;
  }

  public void setZijde( double zijde ) {
    this.zijde = zijde;
    setChanged();
    notifyObservers();
  }
    
  public double getZijde() {
    return zijde;
  }
  
  public double getInhoud() {
    return zijde * zijde * zijde;
  }
  
  public double getOppervlakte() {
    return 6 * zijde * zijde;
  }
}