/* MVC2
 * Tekstview en grafische view, 1 controller voor tekstview
 */

import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.util.*;
import java.awt.*;

public class KubusApplicatie5 extends JFrame {
  public KubusApplicatie5( JPanel view ) {
    setContentPane( view );
    setSize( 375, 375 );
    setTitle( "Kubusapplicatie 5" );
    setDefaultCloseOperation( EXIT_ON_CLOSE );
    setVisible( true );
  }
  
  public static void main( String[] args ) {
    Kubus model = new Kubus( 0 );
    new Controller( model );
  }
}



class Controller implements ActionListener {
  private Kubus model;
  private TekstView tekstView;
  private GrafischeView grafischeView;
  private JFrame frame;
  
  public Controller( Kubus model ) {
    this.model = model;
    tekstView = new TekstView();
    tekstView.setListener( this );
    model.addObserver( tekstView );
    
    grafischeView = new GrafischeView();
    model.addObserver( grafischeView );
    
    JPanel view = new JPanel();
    view.setLayout( new BorderLayout() );

    view.add( tekstView, BorderLayout.SOUTH );
    view.add( grafischeView, BorderLayout.CENTER );
    frame = new KubusApplicatie5( view );
  }
  
  public void actionPerformed( ActionEvent e ) {
    JTextField zijdeVak = (JTextField) e.getSource();
    String s = zijdeVak.getText();
    try {
      double zijde = Double.parseDouble( s );
      model.setZijde( zijde );
    }   
    catch( NumberFormatException ex ) {
      JOptionPane.showMessageDialog(
        frame, "Invoer " + s + " is geen getal",
        "Fout in invoer", JOptionPane.WARNING_MESSAGE );
    }
  }
}



class TekstView extends JPanel implements Observer {
  private JTextField zijdeVak, inhoudVak, oppervlakteVak;

  public TekstView() {
    Box labelBox = Box.createVerticalBox();
    labelBox.add( new JLabel( "Zijde" ) );
    labelBox.add( new JLabel( "Inhoud" ) );
    labelBox.add( new JLabel( "Oppervlakte" ) );

    zijdeVak = new JTextField( 10 );

    inhoudVak = new JTextField( 10 );
    inhoudVak.setEditable( false );

    oppervlakteVak = new JTextField( 10 );
    oppervlakteVak.setEditable( false );
    
    Box tekstvakBox = Box.createVerticalBox();
    tekstvakBox.add( zijdeVak );
    tekstvakBox.add( inhoudVak );
    tekstvakBox.add( oppervlakteVak );
    
    Box totaalBox = Box.createHorizontalBox();
    totaalBox.add( labelBox );
    totaalBox.add( Box.createHorizontalStrut( 20 ) );
    totaalBox.add( tekstvakBox );
    add( totaalBox );
  }

  public void update( Observable o, Object arg ) {
    Kubus model = (Kubus) o; 
    inhoudVak.setText( String.format( "%8.2f", model.getInhoud() ) );
    oppervlakteVak.setText( String.format( "%8.2f", model.getOppervlakte() ) );
  }
  
  public void setListener( ActionListener listener ) {
    zijdeVak.addActionListener( listener );
  }
}

class Kubus extends Observable {
  private double zijde;
  
  public Kubus( double zijde ) {
    this.zijde = zijde;
  }

  public void setZijde( double zijde ) {
    this.zijde = zijde;
    setChanged();
    notifyObservers();
  }
    
  public double getZijde() {
    return zijde;
  }
  
  public double getInhoud() {
    return zijde * zijde * zijde;
  }
  
  public double getOppervlakte() {
    return 6 * zijde * zijde;
  }
}


class GrafischeView extends JPanel implements Observer {
  private int zijde;
  
  public void paintComponent( Graphics g ) {
    super.paintComponent( g );
    if ( zijde > 0) 
      tekenKubus( g );
  }
  
  private void tekenKubus( Graphics g ) {
    int links = (int)( getWidth() - zijde ) / 2;
    int boven = (int)( getHeight() - zijde ) / 2;
    // voorkant kubus
    g.drawRect( links, boven, zijde, zijde );
    // achterkant kubus
    g.setColor( Color.LIGHT_GRAY );
    int deltaX = zijde / 2;
    int deltaY = zijde / 4;
    g.drawRect( links + deltaX, boven - deltaY, zijde, zijde );
    // schuine lijnen
    g.drawLine( links, boven, links + deltaX, boven - deltaY );
    g.drawLine( links + zijde, boven, links + zijde + deltaX, boven - deltaY );
    g.drawLine( links, boven + zijde, links + deltaX, boven + zijde - deltaY );
    g.drawLine( links + zijde, boven + zijde, links + zijde + deltaX, boven + zijde - deltaY );
  }

  public void update( Observable o, Object arg ) {
    Kubus model = (Kubus) o;
    zijde =  (int)model.getZijde();
    repaint();
  }
}

  