/* MVC2
 * Tekstview en grafische view, 2 controllers, 1 voor tekstview en 1 voor grafische view
 */

import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.util.*;
import java.awt.*;

public class KubusApplicatie6 extends JFrame {
  public KubusApplicatie6( JPanel view ) {
    setContentPane( view );
    setSize( 475, 475 );
    setTitle( "Kubusapplicatie 6" );
    setDefaultCloseOperation( EXIT_ON_CLOSE );
    setVisible( true );
  }
  
  public static void main( String[] args ) {
    new Controller( new Kubus( 0 ) );
  }
}


class TekstView extends JPanel implements Observer {
  private JTextField zijdeVak, inhoudVak, oppervlakteVak;

  public TekstView() {
    Box labelBox = Box.createVerticalBox();
    labelBox.add( new JLabel( "Zijde" ) );
    labelBox.add( new JLabel( "Inhoud" ) );
    labelBox.add( new JLabel( "Oppervlakte" ) );

    zijdeVak = new JTextField( 10 );

    inhoudVak = new JTextField( 10 );
    inhoudVak.setEditable( false );

    oppervlakteVak = new JTextField( 10 );
    oppervlakteVak.setEditable( false );
    
    Box tekstvakBox = Box.createVerticalBox();
    tekstvakBox.add( zijdeVak );
    tekstvakBox.add( inhoudVak );
    tekstvakBox.add( oppervlakteVak );
    
    Box totaalBox = Box.createHorizontalBox();
    totaalBox.add( labelBox );
    totaalBox.add( Box.createHorizontalStrut( 20 ) );
    totaalBox.add( tekstvakBox );
    add( totaalBox );
  }

  public void update( Observable o, Object arg ) {
    Kubus model = (Kubus) o;
    zijdeVak.setText( String.format( "%8.2f", model.getZijde() ) );
    inhoudVak.setText( String.format( "%8.2f", model.getInhoud() ) );
    oppervlakteVak.setText( String.format( "%8.2f", model.getOppervlakte() ) );
  }
  
  public void setListener( ActionListener listener ) {
    zijdeVak.addActionListener( listener );
  }
}

class Kubus extends Observable {
  private double zijde;
  
  public Kubus( double zijde ) {
    this.zijde = zijde;
  }

  public void setZijde( double zijde ) {
    this.zijde = zijde;
    setChanged();
    notifyObservers();
  }
    
  public double getZijde() {
    return zijde;
  }
  
  public double getInhoud() {
    return zijde * zijde * zijde;
  }
  
  public double getOppervlakte() {
    return 6 * zijde * zijde;
  }
}


class GrafischeView extends JPanel implements Observer {
  private int zijde;
  
  public void paintComponent( Graphics g ) {
    super.paintComponent( g );
    if ( zijde > 0) 
      tekenKubus( g );
  }
  
  private void tekenKubus( Graphics g ) {
    int links = (int)( getWidth() - zijde ) / 2;
    int boven = (int)( getHeight() - zijde ) / 2;
    // voorkant kubus
    g.drawRect( links, boven, zijde, zijde );
    // achterkant kubus
    g.setColor( Color.LIGHT_GRAY );
    int deltaX = zijde / 2;
    int deltaY = zijde / 4;
    g.drawRect( links + deltaX, boven - deltaY, zijde, zijde );
    // schuine lijnen
    g.drawLine( links, boven, links + deltaX, boven - deltaY );
    g.drawLine( links + zijde, boven, links + zijde + deltaX, boven - deltaY );
    g.drawLine( links, boven + zijde, links + deltaX, boven + zijde - deltaY );
    g.drawLine( links + zijde, boven + zijde, links + zijde + deltaX, boven + zijde - deltaY );
  }

  public void update( Observable o, Object arg ) {
    Kubus model = (Kubus) o;
    zijde =  (int)model.getZijde();
    repaint();
  }
}  
  
  
class Controller extends MouseAdapter 
                 implements ActionListener, MouseMotionListener {
  private Kubus model;
  private TekstView tekstView;
  private GrafischeView grafischeView;
  private JFrame frame;
  
  private boolean opZijde;
  private int startX, startY, startZijde;
  
  public Controller( Kubus model ) {
    this.model = model;
    tekstView = new TekstView();
    tekstView.setListener( this );
    model.addObserver( tekstView );
    
    grafischeView = new GrafischeView();
    grafischeView.addMouseListener( this );
    grafischeView.addMouseMotionListener( this );
    model.addObserver( grafischeView );
    
    JPanel view = new JPanel();
    view.setLayout( new BorderLayout() );

    view.add( tekstView, BorderLayout.SOUTH );
    view.add( grafischeView, BorderLayout.CENTER );
    frame = new KubusApplicatie6( view );
  }
  
  public void actionPerformed( ActionEvent e ) {
    JTextField zijdeVak = (JTextField) e.getSource();
    String s = zijdeVak.getText();
    try {
      double zijde = Double.parseDouble( s );
      model.setZijde( zijde );
    }   
    catch( NumberFormatException ex ) {
      JOptionPane.showMessageDialog(
        frame, "Invoer " + s + " is geen getal",
        "Fout in invoer", JOptionPane.WARNING_MESSAGE );
    }
  }
  
  public void mousePressed( MouseEvent e ) {
    startZijde = (int) model.getZijde();
    int rechts = ( grafischeView.getWidth() + startZijde ) / 2;
    int boven = ( grafischeView.getHeight() - startZijde ) / 2;
    startX = e.getX();
    startY = e.getY();
    
    opZijde = isOpZijde( startX, startY, rechts, boven );
  }

  public void mouseReleased( MouseEvent e ) {
    opZijde = false;
  }

  public void mouseDragged( MouseEvent e ) {
    if( opZijde ) {
      int dX = e.getX() - startX;
      int nieuweZijde = startZijde + dX;
      if( nieuweZijde > 0 )
        model.setZijde( nieuweZijde );
    }
  }
  
  private boolean isOpZijde( int x, int y, int rechts, int boven ) {
    return x >= rechts - 3 && x <= rechts + 3 &&
           y >= boven && y <= boven + model.getZijde();
    
  }

  // niet gebruike methode uit MouseMotionListener
  public void mouseMoved( MouseEvent e ) {}
}

  