import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;

public class View extends JPanel implements Observer {
  private JTextField resultVak, invoerVak;
  private JButton plusKnop, minKnop, undoKnop;
  private Calculator calculator;
  CommandProcessor commandProcessor;
  
  public View( Calculator calculator ) {
    this.calculator = calculator;
    calculator.addObserver( this );
    
    // Maak de gebruikersinterface
    resultVak = new JTextField( 10 );
    resultVak.setBackground( Color.YELLOW );
    invoerVak = new JTextField( 10 );
    plusKnop = new JButton( "+" );
    minKnop  = new JButton( "-" );
    undoKnop = new JButton( "Undo" );
    
    setLayout( new GridLayout( 3, 2 ) );
    add( new JLabel( "Invoer" ) );
    add( invoerVak );
   

    Box hbox = Box.createHorizontalBox();
    hbox.add( plusKnop );
    hbox.add( minKnop );
    hbox.add( undoKnop );
    add( Box.createHorizontalStrut( 10 ) );
    add( hbox );

    add( new JLabel( "Resultaat" ) );
    add( resultVak );

    update( calculator, null );
    
    // Koppel de listeners aan de knoppen
    plusKnop.addActionListener( new PlusKnopListener() );
    minKnop.addActionListener( new MinKnopListener() );
    undoKnop.addActionListener( new UndoKnopListener() );
    
    commandProcessor = new CommandProcessor();
  }
  
  public void update( Observable model, Object o ) {
    Calculator calculator = (Calculator) model;
    resultVak.setText( "" + calculator.getResult() );        
  }

  
  private class PlusKnopListener implements ActionListener {
    private Command command;
    
    public void actionPerformed( ActionEvent e ) {
      int getal = Integer.parseInt( invoerVak.getText() );
      command = new PlusCommand( calculator, getal );
      commandProcessor.execute( command );
    }
  }
  
  private class MinKnopListener implements ActionListener {
    private Command command;
    
    public void actionPerformed( ActionEvent e ) {
      int getal = Integer.parseInt( invoerVak.getText() );
      command = new MinCommand( calculator, getal );
      commandProcessor.execute( command );
    }
  }

  private class UndoKnopListener implements ActionListener {
    public void actionPerformed( ActionEvent e ) {
      commandProcessor.undo();
    }
  }
} 