import java.util.*;

public class Doolhof extends Observable {
  private Veld[][] bord;
  private int aantalRijen, aantalKolommen;
  private Veld spelerveld;
  
  public Doolhof( Veld[][] bord ) {
    this.bord = bord;
    // Leg de afmetingen van het bord vast
    aantalRijen = bord.length;
    aantalKolommen = bord[ 0 ].length;
    // Bepaal het veld waarop de speler staat
    spelerveld = vindSpeler();
  }
  
  public Veld[][] getBord() {
    return bord;
  }
  
  public void doeZet( Richting richting ) {
    Veld buurveld = getBuurveld( spelerveld, richting );
    if( buurveld.isLeeg() )
      verplaatsSpeler( buurveld );
    else
    if( buurveld.heeftKist() ) {
      Veld buurveldVanKist = getBuurveld( buurveld, richting );
      if( buurveldVanKist.isLeeg() ) {
        verplaatsKist( buurveld, buurveldVanKist );
        verplaatsSpeler( buurveld );
      }
    }
  }

  public boolean isSpelKlaar() {
    for( int r = 0; r < aantalRijen; r++ ) {
      for( int k = 0; k < aantalKolommen; k++ ) {
        Veld veld = bord[r][k];
        if( veld.isEindveld() && !veld.heeftKist() )
          return false;
      }
    }
    return true;    
  }

  public Memento getMemento() {
    Veld[][] kopieBord = new Veld[aantalRijen][aantalKolommen];
    for( int r = 0; r < aantalRijen; r++ ) 
      for( int k = 0; k < aantalKolommen; k++ )
        kopieBord[r][k] = (Veld) bord[r][k].clone();
    return new Memento( kopieBord );
  }
  
  public void setMemento( Memento memento ) {
    bord = memento.getBord();
    spelerveld = vindSpeler();
    setChanged();
    notifyObservers();
  }

    
  // Hulpmethoden
  private Veld getVeld( Positie pos ) {
    return bord[ pos.getR() ][ pos.getK() ];
  }

  private Veld getBuurveld( Veld veld, Richting richting ) {
    Positie positie = veld.getPositie();
    Positie buurpositie = positie.getBuurpositie( richting );
    Veld buurveld = getVeld( buurpositie );
    return buurveld;
  }
  
  private void verplaatsKist( Veld vanVeld, Veld naarVeld ) {
    vanVeld.setKist( false );
    naarVeld.setKist( true );
  }

  private void verplaatsSpeler( Veld naarVeld ) {
    spelerveld.setSpeler( false );
    naarVeld.setSpeler( true );
    spelerveld = naarVeld;
    setChanged();
    notifyObservers();
  }
  
  private Veld vindSpeler() {
    for( int r = 0; r < aantalRijen; r++ ) {
      for( int k = 0; k < aantalKolommen; k++ )
        if( bord[r][k].heeftSpeler() )
          return bord[r][k];
    }
    return null;
  }
  
  
  // Inwendige klasse
  public class Memento {
    private Veld[][] bord;
  
    // Constructor 
    private Memento( Veld[][] kopieBord ) {
      this.bord = kopieBord;
    }

    // Methode
    private Veld[][] getBord() {
      return bord;
    }
  }
}
