import java.util.*;

public class Doolhof extends Observable {
  private Veld[][] bord;
  private int aantalRijen, aantalKolommen;
  private Veld spelerveld;
  
  public Doolhof( Veld[][] bord ) {
    this.bord = bord;
    // Leg de afmetingen van het bord vast
    aantalRijen = bord.length;
    aantalKolommen = bord[ 0 ].length;
    // Bepaal het veld waarop de speler staat
    for( int r = 0; r < aantalRijen; r++ ) {
      for( int k = 0; k < aantalKolommen; k++ )
        if( bord[r][k].heeftSpeler() )
          spelerveld = bord[r][k];
    }
  }
  
  public Veld[][] getBord() {
    return bord;
  }
  
  public Memento getMemento( Richting richting ) {
    boolean kanSpelerVerplaatsen = false;
    boolean kanKistVerplaatsen = false;
    Veld buurveld = getBuurveld( spelerveld, richting );
    if( buurveld.isLeeg() ) {
      kanSpelerVerplaatsen = true;
    }
    else
    if( buurveld.heeftKist() ) {
      Veld buurbuurveld = getBuurveld( buurveld, richting );
      if( buurbuurveld.isLeeg() ) {
        kanSpelerVerplaatsen = true;
        kanKistVerplaatsen = true;
      }
    }
    return new Memento( richting, kanSpelerVerplaatsen, kanKistVerplaatsen );
  }
  
  private Veld getVeld( Positie pos ) {
    return bord[ pos.getR() ][ pos.getK() ];
  }
  
  
  // Doe een zet op grond van memento
  public void doeZet( Memento memento ) {
    if( memento.kanSpelerVerplaatsen() ) {
      Richting richting = memento.getRichting();
      if( memento.kanKistVerplaatsen() ) {
        Veld kistveld = getBuurveld( spelerveld, richting );
        verplaatsKist( kistveld, richting );
      }
      verplaatsSpeler( richting );
    }
  }
  
  public void zetTerug( Memento memento ) {
    if( memento.kanSpelerVerplaatsen() ) {
      Richting terug = memento.getRichting().tegenovergestelde();
      if( memento.kanKistVerplaatsen() ) {
        Veld kistveld = getBuurveld( spelerveld, memento.getRichting() );
        verplaatsKist( kistveld, terug );
      }
      verplaatsSpeler( terug );
    }
  }


  public void doeZet( Richting richting ) {
    Veld buurveld = getBuurveld( spelerveld, richting );
    if( buurveld.isLeeg() )
      verplaatsSpeler( buurveld );
    else
    if( buurveld.heeftKist() ) {
      Veld buurveldVanKist = getBuurveld( buurveld, richting );
      if( buurveldVanKist.isLeeg() ) {
        verplaatsKist( buurveld, buurveldVanKist );
        verplaatsSpeler( buurveld );
      }
    }
  }
    
  // Hulpmethoden
  private Veld getBuurveld( Veld veld, Richting richting ) {
    Positie positie = veld.getPositie();
    Positie buurpositie = positie.getBuurpositie( richting );
    Veld buurveld = getVeld( buurpositie );
    return buurveld;
  }
  
  private void verplaatsKist( Veld vanVeld, Veld naarVeld ) {
    vanVeld.setKist( false );
    naarVeld.setKist( true );
  }
  
  private void verplaatsKist( Veld vanVeld, Richting richting ) {
    Veld naarVeld = getBuurveld( vanVeld, richting );
    vanVeld.setKist( false );
    naarVeld.setKist( true );
  }
  
  private void verplaatsSpeler( Veld naarVeld ) {
    spelerveld.setSpeler( false );
    naarVeld.setSpeler( true );
    spelerveld = naarVeld;
    setChanged();
    notifyObservers();
  }
  
  private void verplaatsSpeler( Richting richting ) {
    Veld naarVeld = getBuurveld( spelerveld, richting );
    verplaatsSpeler( naarVeld );
  }
  
  public boolean isSpelKlaar() {
    for( int r = 0; r < aantalRijen; r++ ) {
      for( int k = 0; k < aantalKolommen; k++ ) {
        Veld veld = bord[r][k];
        if( veld.isEindveld() && !veld.heeftKist() )
          return false;
      }
    }
    return true;    
  }
}
